@@ -87,7 +87,8 @@ void DVB::init(char *dvbname, char *siname, int adapt, int min) {
 	struct dvb_frontend_info feinfo;
 
 	int failed =0;
-	 
+	weak=0;
+
 	minor = min;
 	adapter = adapt;
 
@@ -271,14 +272,14 @@ int DVB::SetTP(unsigned int tpid, unsigned int satid)
 		set_diseqc_nb(lnb->diseqcnr);
 		front_param.u.qpsk.symbol_rate = tp->srate;
 		front_param.u.qpsk.fec_inner = (fe_code_rate_t)tp->fec;
-		front_param.inversion = INVERSION_OFF;
+		front_param.inversion = tp->inversion;
 
 		transponder_srate = tp->srate;
 		transponder_pol = tp->pol ? 'H':'V';
 		break;
 	case FE_QAM:
 	        front_param.frequency = tp->freq;
-		front_param.inversion = INVERSION_AUTO;
+		front_param.inversion = tp->inversion;
 		front_param.u.qam.symbol_rate = tp->srate;
 		front_param.u.qam.fec_inner = (fe_code_rate_t)tp->fec;
 		front_param.u.qam.modulation=(fe_modulation_t) (tp->qam+1);
@@ -288,7 +289,7 @@ int DVB::SetTP(unsigned int tpid, unsigned int satid)
 
 	case FE_OFDM:
 	        front_param.frequency = tp->freq;
-		front_param.inversion = INVERSION_OFF;
+		front_param.inversion = tp->inversion;
 		front_param.u.ofdm.bandwidth = (fe_bandwidth_t)tp->band;
 		front_param.u.ofdm.code_rate_HP = (fe_code_rate_t)tp->hp_rate;
 		front_param.u.ofdm.code_rate_LP = (fe_code_rate_t)tp->lp_rate;
@@ -506,10 +507,19 @@ int DVB::parse_pmt(Channel *chan, uint8_t *buf)
 		case 6: 
 			chan->tmppid = epid;
 			break;
-
-		default:
+/*
+		case 5:
+			cerr << "private service 0x" << hex << int(buf[c]) 
+			     << " for service id 0x" << chan->pnr 
+			     << " with pid 0x"<< epid << dec<< endl; 
 			break;
-		}
+		default:
+			cerr << "other service 0x" << hex << int(buf[c]) 
+			     << " for service id 0x" << chan->pnr 
+			     << " with pid 0x"<< epid << dec<< endl; 
+			break;
+*/	
+	}
 		c += 5;
 		if (eslen) c += parse_descriptor(chan, buf+c, eslen);
 	}
@@ -930,13 +940,14 @@ int DVB::SetChannel(Channel *chan,  char* apref, uint16_t *apidp,
 	
 	int c=0;
 	if (chan->pnr)
-		if (chan->vpid == NOPID)
+		if (chan->vpid == NOPID){
+			check_pids(chan);
+			scan=1;
 			while (chan->apidnum == 0 && c<10) {
 				check_pids(chan);
-				scan = 1;
 				c++;
 			}
-
+		}
 	
 	vpid = chan->vpid;
 	if (chan->apidnum){
@@ -997,20 +1008,22 @@ int DVB::SetChannel(Channel *chan,  char* apref, uint16_t *apidp,
 }
 
 int DVB::SetChannel(int chnr, char* apref, uint16_t *apidp, 
-		    uint16_t *vpidp) 
+		    uint16_t *vpidp, bool tune) 
 {
 	if (no_open) return -1;
 
 	if (chnr>=num[CHAN] || chnr < 0)
 	        return -1;
 
-	get_front();
-	if (SetTP(chans[chnr].tpid, chans[chnr].satid) < 0) return -1;
+	if(tune) {
+		get_front();
+		if (SetTP(chans[chnr].tpid, chans[chnr].satid) < 0) return -1;
+	}
 	stop_apid();
 	stop_vpid();
 	stop_ttpid();
 	stop_pcrpid();
-	if (set_front()<0) return -1;
+	 if(tune && (set_front()<0)) return -1;
        
 	return SetChannel(&chans[chnr], apref, apidp, vpidp);
 }
@@ -1560,6 +1573,34 @@ void DVB::bar2(int x, int y, int w, int h, int val, int col1, int col2)
 	}
 }
 
+
+
+int DVB::SetFullFilter(uint16_t pid) 
+{
+	char devname[80];
+	sprintf(devname,DEMUX_DEV,adapter,minor);
+	struct dmx_pes_filter_params pesFilterParams; 
+
+	int fd_section=open(devname, O_RDWR|O_NONBLOCK);
+	
+	if (fd_section < 0) return fd_section;
+	
+	pesFilterParams.input = DMX_IN_FRONTEND; 
+	pesFilterParams.output = DMX_OUT_TS_TAP; 
+	pesFilterParams.pes_type = DMX_PES_OTHER; 
+	pesFilterParams.flags = DMX_IMMEDIATE_START;
+	
+	if (pid == NOPID )pesFilterParams.pid = DMX_FULL_TS_PID;
+	else pesFilterParams.pid = pid;
+	if (ioctl(fd_section, DMX_SET_PES_FILTER, &pesFilterParams) < 0) {
+	        printf("Could not set PES filter\n"); 
+		close(fd_section);
+		return -1;
+	}
+	return fd_section;
+}
+
+
 uint16_t DVB::SetFilter(uint16_t pid, uint16_t section, uint16_t mode) 
 { 
 	struct dmx_sct_filter_params secFilterParams;
@@ -1642,12 +1683,10 @@ int DVB::GetSection(uint8_t *buf, ushort PID, uint8_t sec,
 		loopc++;
 		
 		pid = PID;
-		read(handle, buf, 3);
+		read(handle, buf, MAXSECSIZE);
 		seclen = 0;
 		seclen |= ((buf[1] & 0x0F) << 8); 
 		seclen |= (buf[2] & 0xFF);
-		
-		read(handle, buf+3, seclen);
 		seclen+=3;
 		section=buf[0];
 		sectionnum=buf[6];
@@ -1681,14 +1720,11 @@ int DVB::GetSection(uint8_t *buf,
 			break;
 		loopc++;
 		pid = PID;
-		read(handle, buf, 3);
+		read(handle, buf, MAXSECSIZE);
 		seclen = 0;
 		seclen |= ((buf[1] & 0x0F) << 8); 
 		seclen |= (buf[2] & 0xFF);
-		
-		read(handle, buf+3, seclen);
 		seclen+=3;
-
 		section=buf[0];
 		sectionnum=buf[6];
 		maxsec=buf[7];
@@ -1737,7 +1773,11 @@ void DVB::get_front(void)
 
 int DVB::check_frontend()
 {
-	return chck_frontend(fd_frontend, &festat);
+	if (weak)
+		return weak_chck_frontend(fd_frontend, &festat);
+	else
+		return chck_frontend(fd_frontend, &festat);
+
 }
 
 int DVB::tune_it(struct dvb_frontend_parameters *front_param)
@@ -1754,8 +1794,13 @@ int DVB::tune_it(struct dvb_frontend_parameters *front_param)
 	chk=check_frontend();
 
 	if (!chk){
-		cerr << "Tuning failed" << endl;
-		return -1;
+		weak = 1;
+		chk = check_frontend();
+		weak = 0;
+		if (!chk){
+			cerr << "Tuning failed" << endl;
+			return -1;
+		}
 	}
 
 	return 0;
@@ -2911,12 +2956,13 @@ void set_diseqc(int fdf, int snum, fe_sec_voltage_t v, fe_sec_tone_mode_t t)
         ioctl(fdf, FE_SET_TONE, t);
 }
 
-int tune(int fdf, uint32_t freq, uint32_t sr, fe_code_rate_t fec)
+int tune(int fdf, uint32_t freq, uint32_t sr, fe_code_rate_t fec, 
+	 fe_spectral_inversion_t inv=INVERSION_OFF)
 {
 	struct dvb_frontend_parameters tune;
 		
 	tune.frequency = freq;
-	tune.inversion = INVERSION_OFF;
+	tune.inversion = inv;
 	tune.u.qpsk.symbol_rate = sr;
 	if (!fec)
 		tune.u.qpsk.fec_inner = FEC_AUTO;
@@ -3034,7 +3080,7 @@ struct in_addr getaddress (const char *name)
 	struct in_addr in;
 	struct hostent *hp = gethostbyname (name);
 
-	fprintf(stderr,"Looking up host %s\n", name);
+	//fprintf(stderr,"Looking up host %s\n", name);
 	
 	
 	if (hp)
@@ -3067,7 +3113,7 @@ int tcp_client_connect(const char *hostname, int sckt)
 			exit(1);
 		}
 		
-		fprintf(stderr,"Trying to connect...\n");
+		//fprintf(stderr,"Trying to connect...\n");
 
 		if((dummy = connect(sock, (struct sockaddr *)&name, 
 				    sizeof(name))) == ECONNREFUSED){
@@ -3081,7 +3127,7 @@ int tcp_client_connect(const char *hostname, int sckt)
 		}
 	} while(dummy);
 	
-	fprintf(stderr,"Connection established.\n");
+//	fprintf(stderr,"Connection established.\n");
 	
 	return sock;
 }
@@ -3132,6 +3178,26 @@ void client_send_msg(int fd, uint8_t *msg, int size)
 //	fprintf(stderr,"%d bytes sent.\n",sent);
 }
 
+int weak_chck_frontend (int fefd, frontend_stat *festat)
+{
+	fe_status_t status;
+	int i;
+	
+	for (i = 0; i < 10; i++) {
+		usleep (200000);
+		
+		if (ioctl(fefd, FE_READ_STATUS, &status) == -1) {
+			perror("FE_READ_STATUS failed");
+			return 0;
+		}			
+		
+		if (status & FE_HAS_LOCK)
+			return 1;
+	}
+	
+	return 0;
+}
+
 int chck_frontend (int fefd, frontend_stat *festat)
 {
 	fe_status_t status;
@@ -3165,10 +3231,9 @@ int chck_frontend (int fefd, frontend_stat *festat)
 		ioctl(fefd, FE_READ_BER, &ber);
 		ioctl(fefd, FE_READ_UNCORRECTED_BLOCKS, &u_blocks);
 	} else return 0;
-/*
-	fprintf (stderr, "status %02x | signal %04x | snr %04x | ber %08x | unc %08x | ",
-		 status, strength, snr, ber, u_blocks);
-*/
+//	fprintf (stderr, "status %02x | signal %04x | snr %04x | ber %08x | unc %08x | ",
+//	 status, strength, snr, ber, u_blocks);
+
 	festat->status = status;
 	festat->snr = snr;
 	festat->strength = strength;
@@ -3194,7 +3259,7 @@ int DVB::scan_tp(uint16_t tpid, uint16_t satid, int timeout, int verbose)
 {
 	int seclen=0;
 	uint8_t section, sectionnum=0xff;
-	uint8_t buf[4096];
+	uint8_t buf[MAXSECSIZE];
 	struct pollfd pfd[MAX_PIDS];
 	uint16_t pids[MAX_PIDS];
 	uint16_t pnr[MAX_PIDS];
@@ -3207,8 +3272,12 @@ int DVB::scan_tp(uint16_t tpid, uint16_t satid, int timeout, int verbose)
 	int active = 0;
 	int pidf = 0;
 	uint16_t tsid;
+	int oldcf=0;
 
 	if (timeout < 0) timeout = 5000;
+	for (int i=0; i< num[CHAN]; i++)
+		if (chans[i].tpid == tpid && chans[i].satid == satid)
+			oldcf++;
 
 	memset(pids, 0, MAX_PIDS*sizeof(uint16_t));
 	memset(sec, 0, MAX_PIDS*sizeof(uint8_t));
@@ -3268,22 +3337,10 @@ int DVB::scan_tp(uint16_t tpid, uint16_t satid, int timeout, int verbose)
 			if (np && (pid_state[i] & SCANNING) && (pfd[i].events & POLLIN)){
 				seclen=0;
 			
-				if (read(filter_fds[i], buf, 3) < 3)continue;
+				if (read(filter_fds[i], buf, MAXSECSIZE) < 3)
+					continue;
 				seclen |= ((buf[1] & 0x03) << 8); 
 				seclen |= (buf[2] & 0xFF);
-				
-				int count = 0;
-				int tries = 0;
-				int cr = 0;
-				while (count < seclen && tries < 10){
-					if ( (cr = read(filter_fds[i], buf+3, seclen))<0)
-						perror("Section Filter read");
-					else {
-						count += cr;
-						tries++;
-					}
-				}
-
 				seclen += 3;
 				section = buf[0];
 				sectionnum = buf[6];
@@ -3452,7 +3509,7 @@ int DVB::scan_tp(uint16_t tpid, uint16_t satid, int timeout, int verbose)
 		close(filter_fds[i]);
 	}
 
-	return cf;
+	return cf-oldcf;
 }
 
 int DVB::scan_TP(uint16_t tpid, uint16_t satid, int timeout, int verbose)
